-- Greetings! Here's a script that will allow you to spawn and play with traffic in scenarios. If this looks complicated, don't worry; there will be many detailed comments so that you can try to use this stuff in your own scenarios, too! Lines that say "ESSENTIAL" are the most important lines to copy if you just want to use them for easy traffic setup.

local M = {}

local scenario = require("scenario/scenarios") -- useful for getting the scenario timer, custom finish triggers, and more

local player = {} -- general player object
--local timeLimit = 200 -- scenario time limit
--local waypoints = {"hd_wp1", "hd_wp2", "hd_wp3", "hd_wp4", "hd_wp5", "hd_wp6", "hd_wp7", "hd_wp8", "hd_wp9"} -- list of race waypoint names
--local startPoint = "hd_start" -- optional name of start trigger zone (if you don't have or need one, delete this line)
--local timeLevels = {120, 150, 190, 200} -- list of various goal times to achieve, just for fun
--local text = { -- lines of text to use for scenario fail or win screens
--	win = "Yay, you made it! ",
--	time1 = "Wow, that was insanely speedy! You're not just a delivery driver, you're a top tier racer!",
--	time2 = "Great driving, that was faster than expected. Applause for you!",
--	time3 = "Enjoy the party!",
--	time4 = "The timing was too close, the party was getting quite impatient!",
--	crash = "Well, looks like you're stuck there! That's a fail.",
--	late = "You ran out of time! What a disappointment."
--}

local core_count = Engine.Platform.getCPUInfo().coresPhysical or 4 -- number of user CPU cores; default 4
	if core_count <= 4 then
		quantity = core_count - 1
	end
	if core_count > 4 and core_count <= 6 then
		quantity = core_count
	end
	if core_count > 6 and core_count <= 16 then
		quantity = core_count + 1
	end
	if core_count > 16 then
		quantity = 17
	end
	quantity = clamp(quantity, 3, 17) -- number of traffic vehicles to use; I set the minimum to 3 and the maximum to 7
--local damageLimit = 10000 -- amount of damage that can start the player fail timer
--local win = true -- allows the success message at the end of the scenario
local group = {{"sunburst", "sport_RS_DCT"}, {"van", "deliverytruck"}, {"fullsize", "luxe"}, {"semi", "t75_fifthwheel"}, {"pickup", "crew_heavy_rwd"}, {"hopper", "classic"}, {"bluebuck", "lowrider"}, {"moonhawk", "drag"}, {"van"}, {"legran", "se_v6_towing_M"}, {"super", "390gtr"}, {"citybus", "city"}, {"burnside", "leadsled"}, {"hatch", "custom"}, {"roamer", "cool"}, {"coupe", "drift"}, {"etki", "Mark_1_TMD"}}

local function onScenarioLoaded()
  core_multiSpawn.setGroup("custom", group)
  core_multiSpawn.setupVehicles(quantity, true)
  log("A", "scenario", "Traffic vehicles spawned: "..quantity)
end

--local function onScenarioLoaded()
--	extensions.gameplay_traffic.setupTraffic(quantity, true) -- ESSENTIAL: This function is used to spawn traffic before the scenario is ready to play. It works in a similar way to the radial menu spawns. The full function call is: extensions.gameplay_traffic.setupTraffic(amount, police, limit, autoActive)
	-- amount: the number of traffic vehicles to spawn
	-- police: if true, include police vehicles in the spawn group, default 2 vehicles
	-- limit: if true, the amount is also the maximum limit of traffic vehicles allowed to spawn; i.e., if amount was 5, and there were already 3 cars on the map, it would spawn only 2 cars
	-- autoActive: if true, automatically set all other vehicles as traffic; if false, you can do it manually; see next essential function
--	log("A", "scenario", "Traffic vehicles spawned: "..quantity) -- optional message to display in the debug console; press (~)
--end

local function onScenarioUIReady()
	player.obj = be:getPlayerVehicle(0) -- player vehicle object
	player.id = player.obj:getID() -- player vehicle ID
--	player.failTimer = 0 -- player fail timer; can trigger after a wreck
	
	if startPoint and scenetree.findObject(startPoint) then -- if the start trigger named "hd_start" exists, run the lines below
		local pos = scenetree.findObject(startPoint):getPosition()
		local rot = scenetree.findObject(startPoint):getRotation()
		player.obj:setPositionRotation(pos.x, pos.y, pos.z, rot.x, rot.y, rot.z, rot.w) -- teleport the player vehicle to the start trigger. Why? Traffic always spawns in front of the player vehicle (or camera), and I wanted to move the player away from the initial traffic at the start of the scenario. Useful!
	end
	
	win = true
	
	extensions.gameplay_traffic.activate(true) -- ESSENTIAL: This function activates the traffic mode. If the argument is set to "true", then all other vehicles are automatically used for traffic; otherwise, you can use "extensions.gameplay_traffic.insertTraffic(id)" to manually add vehicles to the traffic mode.
end

-- function onRaceTick(tick)
--	local mainTimer = scenario.getScenario().timer -- main timer
--
--	player.damage = map.objects[player.id].damage -- player damage
--	player.vel = map.objects[player.id].vel:length() -- player velocity, in metres per second
--	
--	if player.damage >= damageLimit then -- if player is damaged enough
--		if player.vel <= 4 then -- if the player is moving slow enough
--			player.failTimer = player.failTimer + tick -- increase the fail timer
--		else
--			player.failTimer = 0 -- reset the fail timer
--		end
--	end
--	
--	if player.failTimer >= 4 then -- if the player is damaged and stuck for more than 4 seconds
--		win = false
--		scenario.finish({failed = text.crash})
--	end
--	
--	if mainTimer >= timeLimit then -- if the time limit is reached
--		win = false
--		scenario.finish({failed = text.late})
--	end
--end

--local function onRaceResult()
--	if win then
--		local finishTime = scenario.getScenario().timer
--		local finishText = text.win
--		local extraText = ""
--		
--		for i, v in ipairs(timeLevels) do -- loop that goes through the values in the timeLevels table
--			if finishTime <= v then -- if the finish time is less than the current value in the table
--				extraText = text["time"..i] -- extra text will match the time level
--				break -- break and exit the loop
--			end
--		end
--		
--		scenario.finish({msg = finishText..extraText})
--	end
--end

M.onScenarioLoaded = onScenarioLoaded
M.onScenarioUIReady = onScenarioUIReady
--M.onRaceTick = onRaceTick
--M.onRaceResult = onRaceResult

return M